#!/usr/bin/python

# basic imports

import os, sys, imp
import time, thread, tempfile, subprocess, signal
import re
import wx

# local imports

import test_problems
import partition_input

# Platforms.  We'll assume GTK, and test for Win32 and Mac when necessary

def Win32():
    return wx.Platform=='__WXMSW__'

def Mac():
    return wx.Platform=='__WXMAC__'

def GTK():
    return wx.Platform=='__WXGTK__'

if Win32():
    import win32api, win32process

Things_to_do = """
To Do:
 + Options Panels.
 + Syntax highlighting in formula boxes (comments, attributes).
 
To Think about:
 + Real-time statistics.
 + Import theories.
"""

def program_dir():
    """
    This gets the full pathname of the directory containing the program.
    It is used for referring to other files (binaries, images, etc.).
    """
    if True:
        return sys.path[0]
    elif (hasattr(sys, 'frozen') or imp.is_frozen('__main__')):
        # running from exe generated by py2exe
        return os.path.dirname(sys.executable)
    else:
        return os.path.dirname(os.path.abspath(sys.argv[0]))

def bin():
    if Win32():
        return 'bin-win32'
    elif Mac():
        return 'bin-mac'
    else:
        return 'bin'

def bin_dir():
    return os.path.join(program_dir(), bin())

def image_dir():
    return os.path.join(program_dir(), 'images')

def sample_dir():
    return os.path.join(program_dir(), 'samples')

def binary_ok(fullpath):
    if not fullpath:
        return False
    elif Win32():
        return os.access(fullpath + '.exe', os.X_OK)
    else:
        return os.access(fullpath, os.X_OK)

def open_dir_style():
    # Mac and Win32 remember directory, even after quitting program;
    # else (GTK), use our
    style = wx.OPEN
    if Mac() or Win32():
        return ('', style)
    else:
        # return (os.getcwd(), wx.OPEN | wx.CHANGE_DIR)
        if Top.current_path:
            return (os.path.dirname(Top.current_path), style)
        else:
            return (os.getcwd(), style)

def saveas_dir_style():
    style = wx.SAVE | wx.OVERWRITE_PROMPT
    if Win32():  # Win32 uses dir remembered from 'open'
        return ('', style)
    elif Mac():  # Mac doesn't use dir remembered from 'open'
        if Top.current_path:
            return (os.path.dirname(Top.current_path), style)
        else:
            return ('', wx.SAVE)
    else:        # GTK doesn't remember
        # return (os.getcwd(), style | wx.CHANGE_DIR)
        if Top.current_path:
            return (os.path.dirname(Top.current_path), style)
        else:
            return (os.getcwd(), style)

class Prover9:

    name = 'Prover9'
    solution_name = 'Proof'
    solution_ext = 'proof'

    logo_path = os.path.join(image_dir(), 'prover9-5a-128t.gif')
    if not os.access(logo_path, os.R_OK):
        print 'Logo %s not found' % logo_path
        logo_path = None

    exits = {}
    exits[0]   = 'Proof'
    exits[1]   = 'Fatal_Error'
    exits[2]   = 'Exhausted'
    exits[3]   = 'Memory_Limit'
    exits[4]   = 'Time_Limit'
    exits[5]   = 'Given_Limit'
    exits[6]   = 'Kept_Limit'
    exits[7]   = 'Action_Exit'
    exits[101] = 'Interrupted'
    exits[102] = 'Crashed'
    exits[-9]  = 'Killed' # Linux, Mac
    exits[-1]  = 'Killed' # Win32
        
    def search_command(self):
        fullpath = os.path.join(bin_dir(), 'prover9')
        if not binary_ok(fullpath):
            return None
        else:
            return [fullpath]

    def success_command(self):
        fullpath = os.path.join(bin_dir(), 'prooftrans')
        if not binary_ok(fullpath):
            return None
        else:
            return [fullpath, 'parents_only']

    def exit_message(self, code):
        if code in self.exits.keys():
            return self.exits[code]
        else:
            return 'unknown exit code: %d' % code

class Mace4:

    name = 'Mace4'
    solution_name = 'Counterexample/Model'
    solution_ext = 'model'

    logo_path = os.path.join(image_dir(), 'mace4-90t.gif')
    if not os.access(logo_path, os.F_OK):
        print 'Logo %s not found' % logo_path
        logo_path = None

    exits = {}
    exits[0]   = 'Conterexample/Model'
    exits[1]   = 'Fatal_Error'
    exits[2]   = 'Search_Exhausted_(no_models)'
    exits[3]   = 'All_Models_(with_models)'
    exits[4]   = 'Time_Limit_(with_models)'
    exits[5]   = 'Time_Limit_(no_models)'
    exits[6]   = 'Memory_Limit_(with_models)'
    exits[7]   = 'Memory_Limit_(without_models)'
    exits[101] = 'Interrupted'
    exits[102] = 'Crashed'
    exits[-9]  = 'Killed' # Linux, Mac
    exits[-1]  = 'Killed' # Win32
 
    def search_command(self):
        fullpath = os.path.join(bin_dir(), 'mace4')
        if not binary_ok(fullpath):
            return None
        else:
            return [fullpath, '-c']

    def success_command(self):
        fullpath = os.path.join(bin_dir(), 'interpformat')
        if not binary_ok(fullpath):
            return None
        else:
            return [fullpath, 'tabular']

    def exit_message(self, code):
        if code in self.exits.keys():
            return self.exits[code]
        else:
            return 'unknown exit code: %d' % code

class State:
    ready     = 0
    running   = 1
    suspended = 2
    done      = 3
    error     = 4

def error_dialog(message):
    dlg = wx.MessageDialog(None, message, '', wx.OK | wx.ICON_ERROR)
    dlg.ShowModal()
    dlg.Destroy()

class Invoke_event(wx.PyEvent):
    """
    This class is used so that a side thread can invoke
    a function in the main GUI thread.
    """
    my_EVT_INVOKE = wx.NewEventType()

    def __init__(self, func, args, kwargs):
        wx.PyEvent.__init__(self)
        self.SetEventType(Invoke_event.my_EVT_INVOKE)
        self.__func = func
        self.__args = args
        self.__kwargs = kwargs

    def invoke(self):
        self.__func(*self.__args, **self.__kwargs)

# END class Invoke_event(PyEvent)

class Busy_bar(wx.Gauge):

    def __init__(self, parent, width=200, height=16,
                 pixels_to_move=2, fill=.9, delay=200):
        self.parent = parent

        wx.Gauge.__init__(self, parent, range=100, size=(width,height))

        self.delay = delay
        self.range = range
        self.state = State.ready
        self.position = 40
        self.direction = 1

    def update_bar(self, evt):
        # This is run periodically, triggered by the timer in self.start().
        self.position += self.direction
        if self.position >= 60:
            self.direction = -1
        elif self.position <= 40:
            self.direction = 1

        self.SetValue(self.position)

    def start(self):
        self.timer = wx.Timer(self, -1)
        wx.EVT_TIMER(self, self.timer.GetId(), self.update_bar)
        self.timer.Start(self.delay)
        self.state = State.running

    def pause(self):
        self.timer.Stop()
        self.state = State.suspended

    def resume(self):
        self.start()
        
    def stop(self):
        self.timer.Stop()
        self.SetValue(0)
        self.position = 40
        self.direction = 1
        self.state = State.ready

# END class Busy_bar(Guage)

class Text_frame(wx.Frame):
    def __init__(self, parent, font, title, text, extension=None, saveas=True):
        wx.Frame.__init__(self, parent, title=title, size=(800,700))

        self.extension = extension

        if saveas:
            saveas_btn = wx.Button(self, -1, 'Save as...')
            self.Bind(wx.EVT_BUTTON, self.saveas, saveas_btn)

        close_btn = wx.Button(self, -1, 'Close')
        self.Bind(wx.EVT_BUTTON, self.close, close_btn)

        self.txt = wx.TextCtrl(self,style=wx.TE_MULTILINE|wx.TE_READONLY|wx.HSCROLL)
        self.txt.SetFont(font)
        
        self.txt.AppendText(text)
        self.txt.ShowPosition(0)
 
        sub_sizer = wx.BoxSizer(wx.HORIZONTAL)
        if saveas:
            sub_sizer.Add(saveas_btn, 0, wx.ALL, 3)
        sub_sizer.Add((0,0), 1)  # strechable space
        sub_sizer.Add(close_btn, 0, wx.ALL, 3)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(sub_sizer, 0, wx.ALL|wx.GROW, 3)
        sizer.Add(self.txt, 1, wx.ALL|wx.GROW, 3)
        self.SetSizer(sizer)

    def saveas(self, evt):
        (dir,style) = saveas_dir_style()  # depends on platform

        if Top.current_path and self.extension:
            dfile = os.path.basename(Top.current_path)
            dfile = re.sub('\.[^.]*$', '', dfile)  # get rid of any extension
            dfile = '%s.%s' % (dfile,self.extension)    # append new extension
        else:
            dfile = ''

        dlg = wx.FileDialog(self, message='Save file as ...',
                            defaultDir=dir, defaultFile=dfile, style=style)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()      # full path
            try:
                f = open(path, 'w')
                f.write(self.txt.GetValue())
                # Do not update Top.current_path
            except IOError, e:
                error_dialog('Error opening file %s for writing.' % path)

        dlg.Destroy()

    def close(self, evt):
        self.Close()
        
# END class Text(Frame)

class Run_program:
    def __init__(self, parent, program, input):
        self.parent = parent
        self.program = program
        self.input = input
        self.output = None
        self.solution = None
        self.exit_code = None
        self.state = State.ready

        thread.start_new_thread(self.run, ())

    def run(self):
        #
        # DO NOT DO ANY GUI STUFF IN HERE, BECAUSE THIS
        # WILL RUN IN A SEPARATE THREAD!!!
        #
        search_command  = self.program.search_command()
        success_command = self.program.success_command()

        if not search_command or not success_command:
            self.state = State.error
        else:
            # use files to avoid buffering problems  (maybe improve later)
            fin  = tempfile.TemporaryFile('w+b')  # stdin
            fout = tempfile.TemporaryFile('w+b')  # stdout
            ferr = tempfile.TemporaryFile('w+b')  # stderr
            fin.write(self.input)
            fin.seek(0)

            if Win32():
                # creationflag says not to pop a DOS box
                self.process = subprocess.Popen(
                    search_command, stdin=fin,
                    stdout=fout, stderr=ferr,
                    creationflags=win32process.CREATE_NO_WINDOW)
            else:
                self.process = subprocess.Popen(
                    search_command, stdin=fin,
                    stdout=fout, stderr=ferr)
                
            self.state = State.running
            self.exit_code = self.process.wait()
            fout.seek(0)  # rewind
            self.output = fout.read()
            if self.exit_code == 0:
                fout.seek(0)
                fout2 = tempfile.TemporaryFile('w+b')
                ferr2 = tempfile.TemporaryFile('w+b')
                p = subprocess.Popen(success_command, stdin=fout,
                                     stdout=fout2, stderr=ferr2)
                rc = p.wait()
                if (rc == 0):
                    fout2.seek(0)
                    self.solution = fout2.read()
                else:
                    self.solution = 'There was an error extracting the solution.'
                fout2.close()
                ferr2.close()
            fin.close()
            fout.close()
            ferr.close()
            self.state = State.done

        self.parent.invoke_later(self.parent.done_with_job)

    def pause(self):
        if self.state == State.running:
            p = subprocess.Popen(['kill', '-STOP', str(self.process.pid)])
            self.state = State.suspended

    def resume(self):
        if self.state == State.suspended:
            p = subprocess.Popen(['kill', '-CONT', str(self.process.pid)])
            self.state = State.running

    def kill(self):
        if self.state == State.running or self.state == State.suspended:
            # Cleanup will occur before the 'run' thread terminates.
            if Win32():
                win32api.TerminateProcess(int(self.process._handle), -1)
            else:
                os.kill(self.process.pid, signal.SIGKILL)
                # p = subprocess.Popen(['kill','-KILL', str(self.process.pid)])

# end class Run_program()
    
class Input_panel(wx.Panel):

    def __init__(self, parent, font, title='Input'):
        self.parent = parent
        wx.Panel.__init__(self, parent)

        text = wx.StaticText(self, -1, title + ':')
        wff_btn = wx.Button(self, -1, 'Well Formed?')
        wff_btn.SetToolTipString('Check syntax of ' + title)
        self.Bind(wx.EVT_BUTTON, self.well_formed_check, wff_btn)
        clear_btn = wx.Button(self, -1, 'Clear')
        clear_btn.SetToolTipString('Clear ' + title)
        self.Bind(wx.EVT_BUTTON, self.clear, clear_btn)

        sub_sizer = wx.BoxSizer(wx.HORIZONTAL)
        sub_sizer.Add(text, 1, wx.ALIGN_BOTTOM, 0)
        sub_sizer.Add(wff_btn, 0, wx.ALL, 3)
        sub_sizer.Add(clear_btn, 0, wx.ALL, 3)

        # ed = editor.Editor(self, -1, style=wx.SUNKEN_BORDER)  # bad on Mac
        self.ed = wx.TextCtrl(self, style=wx.TE_MULTILINE|wx.HSCROLL)
        self.ed.SetFont(font)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(sub_sizer, 0, wx.ALL|wx.GROW, 3)
        sizer.Add(self.ed, 1, wx.ALL|wx.GROW, 3)
        self.SetSizer(sizer)
        self.title = title

    def well_formed_check(self, evt):
        dlg = wx.MessageDialog(self, 'Not implemented yet', '',
                               wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()

    def clear(self, evt):
        self.ed.Clear()

# END class Input_panel(Panel)

class P9_options_panel(wx.Panel):
    def __init__(self, parent):
        self.parent = parent
        wx.Panel.__init__(self, parent)

        panel1 = wx.Panel(self)
        # panel1.SetBackgroundColour('grey80')

        self.type = ['Basic Options', 'All Options']

        heading = wx.StaticText(panel1, -1, 'Prover9 Options')

        self.rb1 = wx.RadioBox(panel1, -1, '', choices=self.type,
                          majorDimension = 1, style=wx.RA_SPECIFY_COLS)

        self.rb1.SetToolTip(wx.ToolTip('Show only basic options or enable '
                                       'access to all options.'))

        self.groups = ['Meta Options','Term Ordering','Limits','Search Prep',
                       'Select Given','Inference Rules','Weighting',
                       'Process Inferred','Input/Output','Goals/Denials',
                       'Hints','Other Options']

        self.rb2 = wx.RadioBox(panel1, -1, 'Option Groups',
                               choices=self.groups,
                               majorDimension = 1, style=wx.RA_SPECIFY_COLS)
        self.rb2.SetToolTip(wx.ToolTip('These are all of the Prover9 '
                                       'options, separated into groups.'))

        self.rb2.Enable(False)

        self.Bind(wx.EVT_RADIOBOX, self.handle_rb1, self.rb1)
        self.Bind(wx.EVT_RADIOBOX, self.handle_rb2, self.rb2)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add((1,10), 0)
        sizer.Add(heading, 0, wx.LEFT|wx.RIGHT|wx.GROW, 10)
        sizer.Add((1,5), 0)
        sizer.Add(self.rb1, 0, wx.LEFT|wx.RIGHT|wx.GROW, 10)
        sizer.Add((1,10), 0)
        sizer.Add(self.rb2, 0, wx.LEFT|wx.RIGHT|wx.GROW, 10)
        panel1.SetSizer(sizer)

        temp = wx.StaticText(self, -1,"""
        Some day, the options in the selected group will be here.""")

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(panel1, 0, wx.ALL|wx.GROW, 3)
        sizer.Add(temp, 1, wx.ALL|wx.GROW, 3)
        self.SetSizer(sizer)

    def handle_rb1(self, evt):
        item = evt.GetInt()
        if self.type[item] == 'Basic Options':
            self.rb2.Enable(False)
        else:
            self.rb2.Enable(True)
        self.Layout()         # rearrange children

    def handle_rb2(self, evt):
        item = evt.GetInt()
        # print 'RadioBox item: ' + self.groups[item]

# END class P9_options_panel(Panel)

class Setup_tabs(wx.Notebook):
    def __init__(self, parent):
        self.parent = parent
        wx.Notebook.__init__(self, parent,
                             style=wx.NO_BORDER|wx.FULL_REPAINT_ON_RESIZE)

        temp = wx.StaticText(self, -1, """
        Some day, this panel will be used for options
        that affect parsing/printing and meanings
        of symbols.  These will apply to both Prover9
        and Mace4.  For example,

            op(400, infix_right, *).
            set(prolog_style_variables).
            redeclare(conjunction, AND).
        """)
        self.AddPage(temp, 'Language Options')

        split = wx.SplitterWindow(self, -1)
        self.assumps = Input_panel(split, Top.box_font, 'Assumptions')
        self.goals   = Input_panel(split, Top.box_font, 'Goals')

        if Mac() or Win32():
            # neg supposed to apply to bottom, but it doesn't work
            sash_position = 350
        else:
            sash_position = -150
                                 
        split.SplitHorizontally(self.assumps, self.goals, sash_position)
        split.SetSashGravity(0.9)
        split.SetMinimumPaneSize(100)
        self.AddPage(split, 'Formulas')

        self.additional = Input_panel(self, Top.box_font,
                                      'Additional_Prover9_Input')
        self.AddPage(self.additional, 'Additional Input')

        self.p9_options = P9_options_panel(self)  # ok everywhere
        self.AddPage(self.p9_options, 'Prover9 Options')

        temp = wx.StaticText(self, -1, """
        Some day, this panel will be used for setting Mace4 options.""")
        self.AddPage(temp, 'Mace4 Options')

        self.SetSelection(1)  # Start with second page (Formulas) showing

    def reset(self):
        self.assumps.ed.Clear()
        self.goals.ed.Clear()
        self.additional.ed.Clear()
        # Should also reset options here.

    def update_font(self, font):
        self.assumps.ed.SetFont(font)
        self.goals.ed.SetFont(font)
        self.additional.ed.SetFont(font)
        if Mac():
            self.Show(False)
            self.Show(True)

    def assemble_input(self):
        other = self.additional.ed.GetValue()
        assumps = self.assumps.ed.GetValue()
        goals = self.goals.ed.GetValue()
        input = ('%s\nformulas(assumptions).\n\n%s\nend_of_list.\n\n'
                 'formulas(goals).\n\n%s\nend_of_list.\n\n'
                 % (other,assumps,goals))
        input = re.sub('\n\s*\n', '\n\n', input)  # collapse blank lines
        return input

    def store_new_input(self, input):
        (assumps, goals, additional) = partition_input.partition(input)

        self.reset()

        self.assumps.ed.AppendText(assumps)
        self.assumps.ed.ShowPosition(0)
        self.goals.ed.AppendText(goals)
        self.goals.ed.ShowPosition(0)
        self.additional.ed.AppendText(additional)
        self.additional.ed.ShowPosition(0)

# END class Setup_tabs(Notebook)
                                       
class Program_panel(wx.Panel):

    def __init__(self, parent, program):
        self.parent = parent
        self.program = program
        self.job = None

        wx.Panel.__init__(self, parent)
        self.Connect(-1, -1, Invoke_event.my_EVT_INVOKE, self.onInvoke)

        # Surrounding box

        box_program = wx.StaticBox(self, -1, program.solution_name + ' Search')
        box_sizer = wx.StaticBoxSizer(box_program, wx.VERTICAL)

        # Logo
        
        if program.logo_path:
            logo_bitmap = wx.Image(program.logo_path,
                                   wx.BITMAP_TYPE_GIF).ConvertToBitmap()
            logo = wx.StaticBitmap(self, -1, logo_bitmap)
        else:
            error_dialog('%s Logo not found.' % program.name)
            logo = None

        # Time limit

        time_sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.time_ctrl = wx.SpinCtrl(self,-1,min=-1,max=1000000,size=(100,-1))
        self.time_ctrl.SetValue(60)
        label = wx.StaticText(self, -1, 'Time Limit: ')
        self.time_ctrl.SetToolTipString('A value of -1 means there is no limit.')
        self.Bind(wx.EVT_SPINCTRL, self.time_limit_change, self.time_ctrl)
        time_sizer.Add(label, 0, wx.ALL|wx.ALIGN_CENTER, 1)
        time_sizer.Add(self.time_ctrl, 0, wx.ALL, 1)
        time_sizer.Add(wx.StaticText(self, -1, 'seconds.'), 0,
                       wx.ALL|wx.ALIGN_CENTER, 1)

        # Start, Pause, Kill

        self.start_btn = wx.Button(self, -1, 'Start')
        self.start_btn.SetToolTipString(
            'Start %s with the current input.' % program.name)
        self.Bind(wx.EVT_BUTTON, self.start, self.start_btn)
        self.pause_btn = wx.Button(self, -1, 'Pause')
        self.pause_btn.SetToolTipString('Pause or Resume %s.' % program.name)
        self.pause_btn.Enable(False)
        self.Bind(wx.EVT_BUTTON, self.pause_resume, self.pause_btn)
        if Win32():
            self.pause_btn.Show(False)
        self.kill_btn = wx.Button(self, -1, 'Kill')
        self.kill_btn.SetToolTipString('Kill %s process.' % program.name)
        self.kill_btn.Enable(False)
        self.Bind(wx.EVT_BUTTON, self.kill, self.kill_btn)

        run_sizer = wx.BoxSizer(wx.HORIZONTAL)
        run_sizer.Add(self.start_btn, 0, wx.ALL, 1)
        run_sizer.Add(self.pause_btn, 0, wx.ALL, 1)
        run_sizer.Add(self.kill_btn, 0, wx.ALL, 1)

        # Busy bar

        self.bar = Busy_bar(self, width=200, height=16, delay=100)

        # Status line

        self.status_text = wx.StaticText(self, -1, 'Ready')
        self.show_save_btn = wx.Button(self, -1, 'Show/Save')
        self.Bind(wx.EVT_BUTTON, self.show_save, self.show_save_btn)
        status_sizer = wx.BoxSizer(wx.HORIZONTAL)
        status_sizer.Add(wx.StaticText(self, -1, 'Status: '), 0, wx.ALL, 1)
        status_sizer.Add(self.status_text, 0, wx.ALL, 1)

        # Show/Save, Reset
        
        self.show_save_btn.Enable(False)
        self.reset_btn = wx.Button(self, -1, 'Reset')
        self.reset_btn.SetToolTipString('Enable another %s search.' % program.name)
        self.Bind(wx.EVT_BUTTON, self.onReset, self.reset_btn)
        self.reset_btn.Enable(False)
        show_sizer = wx.BoxSizer(wx.HORIZONTAL)
        show_sizer.Add(self.show_save_btn, 0, wx.ALL, 1)
        show_sizer.Add(self.reset_btn, 0, wx.ALL, 1)

        # Overall Layout

        if logo:
            box_sizer.Add(logo,     0, wx.ALL|wx.ALIGN_CENTER,5)
        box_sizer.Add(time_sizer,   0, wx.ALL, 5)
        box_sizer.Add(run_sizer,    0, wx.ALL|wx.ALIGN_CENTER,5)
        box_sizer.Add(self.bar,     0, wx.ALL|wx.ALIGN_CENTER,5)
        box_sizer.Add(status_sizer, 0, wx.ALL, 5)
        box_sizer.Add(show_sizer,   0, wx.ALL|wx.ALIGN_CENTER, 5)
        self.SetSizer(box_sizer)

    # Methods

    def time_limit_change(self, evt): 
        # print 'time limit: %d' % self.time_ctrl.GetValue()
        pass

    def start(self, evt):
        self.start_btn.Enable(False)
        self.time_ctrl.Enable(False)
        self.pause_btn.Enable(True)
        self.kill_btn.Enable(True)
        self.show_save_btn.Enable(False)
        self.bar.start()
        self.status_text.SetLabel('Running')

        input = Top.setup.assemble_input()
        # following is temporary, until options panels are working
        input += '\nassign(max_seconds, %d).\n' % self.time_ctrl.GetValue()
        self.job = Run_program(self, self.program, input)

    def pause_resume(self, evt):
        # assume job is running or suspended
        if self.job.state == State.running:
            self.job.pause()
            self.bar.pause()
            self.pause_btn.SetLabel('Resume')
            self.status_text.SetLabel('Paused')
        elif self.job.state == State.suspended:
            self.job.resume()
            self.bar.resume()
            self.pause_btn.SetLabel('Pause')
            self.status_text.SetLabel('Running')
        else:
            error_dialog('pause_resume: bad job state')

    def kill(self, evt):
        # assume job is running or suspended
        self.job.kill()  # calls done_with_job indirectly

    def done_with_job(self):
        self.bar.stop()
        self.start_btn.Enable(False)
        self.pause_btn.Enable(False)
        self.pause_btn.SetLabel('Pause')
        self.kill_btn.Enable(False)
        self.show_save_btn.Enable(True)
        self.reset_btn.Enable(True)

        if self.job.state == State.error:
            self.status_text.SetLabel('Program_Not_Found')
            error_dialog('%s binaries not found, looking in\n%s' %
                         (self.program.name, bin_dir()))
        else:
            message = self.program.exit_message(self.job.exit_code)
            self.status_text.SetLabel(message)

            if self.job.exit_code == 0:
                frame = Text_frame(
                    self, Top.box_font,
                    self.program.name + ' ' + self.program.solution_name,
                    self.job.solution,
                    extension = self.program.solution_ext, saveas=True)
                frame.Show(True)
            else:
                dlg = wx.MessageDialog(
                    self,
                    '%s Exit: %s' % (self.program.name, message),
                    '%s Search Finished' % self.program.name,
                    wx.OK | wx.ICON_INFORMATION)
                dlg.ShowModal()
                dlg.Destroy()

    def onReset(self, evt):
        # assume job exists
        del self.job  # free memory associated with the job
        self.job = None
        self.start_btn.Enable(True)
        self.time_ctrl.Enable(True)
        self.pause_btn.Enable(False)
        self.pause_btn.SetLabel('Pause')
        self.kill_btn.Enable(False)
        self.show_save_btn.Enable(False)
        self.reset_btn.Enable(False)
        self.status_text.SetLabel('Ready')

    def show_save(self, evt):
        menu = wx.Menu()

        id = wx.NewId()
        menu.Append(id, self.program.name + ' Input from Previous Search')
        self.Bind(wx.EVT_MENU, self.ss_input, id=id)

        id = wx.NewId()
        menu.Append(id, self.program.name + ' Output')
        self.Bind(wx.EVT_MENU, self.ss_output, id=id)
        if self.job.state == State.error:
            menu.Enable(id, False)

        id = wx.NewId()
        menu.Append(id, self.program.name + ' ' + self.program.solution_name)
        self.Bind(wx.EVT_MENU, self.ss_solution, id=id)
        if self.job.exit_code != 0:
            menu.Enable(id, False)

        self.PopupMenu(menu)
        menu.Destroy()

    def ss_input(self, evt):
        frame = Text_frame(self, Top.box_font,
                           self.program.name + ' Input from Previous Search',
                           self.job.input,
                           extension='in', saveas=True)
        frame.Show(True)
        
    def ss_output(self, evt):
        frame = Text_frame(self, Top.box_font,
                           self.program.name + ' Output',
                           self.job.output,
                           extension='out', saveas=True)
        frame.Show(True)
        
    def ss_solution(self, evt):
        frame = Text_frame(self, Top.box_font,
                           self.program.name+' '+self.program.solution_name,
                           self.job.solution,
                           extension=self.program.solution_ext, saveas=True)
        frame.Show(True)

    # The following two methods allow GUI events in the main thread
    # to be initiated by other threads.  See class Invoke_event and
    # the Connect statement in the constructor of this class.

    def onInvoke(self, evt):
        evt.invoke()

    def invoke_later(self, func, *args, **kwargs):
        self.GetEventHandler().AddPendingEvent(Invoke_event(func,args,kwargs))

# END class Program_panel(Panel)

class Control_panel(wx.Panel):
    def __init__(self, parent):
        self.parent = parent
        wx.Panel.__init__(self, parent, style=wx.FULL_REPAINT_ON_RESIZE)

        # Show Input Button

        self.show_input_btn = wx.Button(self, -1, 'Show Current Input')
        self.show_input_btn.SetToolTipString(
            'Show the data in the setup panel as a text file.\n'
            'This is what will be given to Prover9 and/or Mace4.\n'
            'This need not be saved before running Prover9 or Mace4.')
        self.Bind(wx.EVT_BUTTON, self.show_input, self.show_input_btn)

        # Program Panels

        self.prover9 = Program_panel(self, Prover9())
        self.mace4 = Program_panel(self, Mace4())

        # Overall Layout

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.show_input_btn, 0, wx.ALL|wx.ALIGN_CENTER, 5)
        sizer.Add((1,10), 1)
        sizer.Add(self.prover9, 0, wx.GROW, 5)
        sizer.Add((1,10), 1)
        sizer.Add(self.mace4, 0, wx.GROW, 5)
        sizer.Add((1,10), 1)
        self.SetSizer(sizer)
        sizer.Fit(self)

    def show_input(self, evt):
        input = Top.setup.assemble_input()
        frame = Text_frame(self, Top.box_font,
                           'Current Input', input,
                           extension='in', saveas=False)
        frame.Show(True)

# END class Control_panel(Panel)

class Main_frame(wx.Frame):
    """
    This is the primary Frame.
    """
    def __init__(self, parent, title):
        self.parent = parent
        wx.Frame.__init__(self, parent, -1, title, pos=(0,36))
        self.saved_client_size = None
        self.current_path = None

        global Top; Top = self  # set global variable for easy access

        # self.SetBackgroundColour(wx.NamedColor('GREY50'))

        self.box_font = wx.Font(12, wx.FONTFAMILY_MODERN,
                                wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL)

        # Create the menubar
        menu_bar = wx.MenuBar()

        # File menu
        self.fmenu = wx.Menu()
        submenu = self.sample_menu()
        self.fmenu.AppendMenu(-1, 'Wired-in Samples', submenu)
        id = wx.NewId()
        self.fmenu.Append(id, 'Browse Samples...')
        self.Bind(wx.EVT_MENU, self.browse_samples, id=id)
        self.fmenu.AppendSeparator()
        id = wx.NewId()
        self.fmenu.Append(id, 'Clear Setup Panel')
        self.Bind(wx.EVT_MENU, self.clear_setup, id=id)
        self.fmenu.Append(wx.ID_OPEN, '&Open Input File...\tCtrl+O')
        self.fmenu.AppendSeparator()
        self.fmenu.Append(wx.ID_SAVE, '&Save Input\tCtrl+S')
        self.fmenu.Enable(wx.ID_SAVE, False)
        self.fmenu.Append(wx.ID_SAVEAS, 'Save Input as...')
        self.fmenu.AppendSeparator()
        self.fmenu.Append(wx.ID_EXIT, '&Quit\tCtrl+Q')
        self.Bind(wx.EVT_MENU, self.open, id=wx.ID_OPEN)
        self.Bind(wx.EVT_MENU, self.save, id=wx.ID_SAVE)
        self.Bind(wx.EVT_MENU, self.saveas, id=wx.ID_SAVEAS)
        self.Bind(wx.EVT_MENU, self.ready_to_close, id=wx.ID_EXIT)
        menu_bar.Append(self.fmenu, '&File')

        # Preferences menu
        self.pref_menu = wx.Menu()
        id = wx.NewId()
        self.pref_menu.Append(id, 'Font for Text Boxes...')
        self.Bind(wx.EVT_MENU, self.select_font, id=id)
        self.pref_menu.AppendSeparator()
        self.tooltip_id = wx.NewId()
        self.pref_menu.Append(self.tooltip_id, 'Show Tool Tips', '',
                              wx.ITEM_CHECK)
        self.pref_menu.Check(self.tooltip_id, True)
        self.Bind(wx.EVT_MENU, self.tooltip_toggle, id=self.tooltip_id)
        menu_bar.Append(self.pref_menu, '&Preferences')

        # View menu
        self.view_menu = wx.Menu()
        self.view_id = wx.NewId()
        self.view_menu.Append(self.view_id, 'Hide Setup Panel')
        self.Bind(wx.EVT_MENU, self.setup_toggle, id=self.view_id)
        self.run_id = wx.NewId()
        self.view_menu.Append(self.run_id,'Hide Run Panel')
        self.Bind(wx.EVT_MENU, self.run_toggle, id=self.run_id)
        menu_bar.Append(self.view_menu, '&View')

        # Help menu
        menu = wx.Menu()
        menu.Append(wx.ID_HELP_CONTENTS, 'Help Contents\tF1')
        menu.AppendSeparator()
        menu.Append(wx.ID_ABOUT, 'About')
        self.Bind(wx.EVT_MENU, self.get_help, id=wx.ID_HELP_CONTENTS)
        self.Bind(wx.EVT_MENU, self.get_about, id=wx.ID_ABOUT)
        menu_bar.Append(menu, '&Help')

        # set Menubar and statusbar
        self.SetMenuBar(menu_bar)
        # self.CreateStatusBar()

        self.setup = Setup_tabs(self)
        self.control = Control_panel(self)

        self.setup.assumps.ed.AppendText(
            test_problems.problems[test_problems.default][0])
        self.setup.goals.ed.AppendText(
            test_problems.problems[test_problems.default][1])
        self.setup.assumps.ed.ShowPosition(0)
        self.setup.goals.ed.ShowPosition(0)

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(self.setup, 1, wx.ALL|wx.GROW, 1)
        sizer.Add(self.control, 0, wx.ALL|wx.GROW, 3)
        self.SetSizer(sizer)
        sizer.Fit(self)

    def sample_menu(self):
        probs = test_problems.problems
        menu = wx.Menu()
        self.probs = {}
        for name in test_problems.problems.keys():
            id = wx.NewId()
            self.probs[id] = name
            menu.Append(id, name)
            self.Bind(wx.EVT_MENU, self.load_sample, id=id)
        return menu

    def load_sample(self, evt):
        prob_name = self.probs[evt.GetId()]
        assumps = test_problems.problems[prob_name][0]
        goals   = test_problems.problems[prob_name][1]
        self.setup.assumps.ed.Clear()
        self.setup.assumps.ed.AppendText(assumps)
        self.setup.assumps.ed.ShowPosition(0)
        self.setup.goals.ed.Clear()
        self.setup.goals.ed.AppendText(goals)
        self.setup.goals.ed.ShowPosition(0)
        # ??? self.setup.additional.ed.Clear()
        self.SetTitle('Prover9/Mace4')

    def ready_to_close(self, evt):
        self.Close()

    def browse_samples(self, evt):
        if not os.access(sample_dir(), os.R_OK):
            error_dialog('The samples directory %s seems to be missing' %
                         sample_dir())
        else:
            (dir,style) = open_dir_style()  # depends on platform
            dlg = wx.FileDialog(self, message='Select a file',
                                defaultDir=sample_dir(), style=style)
            if dlg.ShowModal() == wx.ID_OK:
                path = dlg.GetPath()      # full path
                try:
                    f = open(path)
                    input = f.read()
                    self.setup.store_new_input(input)
                except IOError, e:
                    error_dialog('Error opening file %s for reading.' % path)
            dlg.Destroy()

    def clear_setup(self, evt):
        self.setup.reset()

    def open(self, evt):
        (dir,style) = open_dir_style()  # depends on platform
        dlg = wx.FileDialog(self, message='Select a file',
                            defaultDir=dir, style=style)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()      # full path
            try:
                f = open(path)
                input = f.read()
                self.setup.store_new_input(input)
                self.current_path = path
                self.fmenu.Enable(wx.ID_SAVE, True)
                self.SetTitle(os.path.basename(path) + ' - Prover9/Mace4')
            except IOError, e:
                error_dialog('Error opening file %s for reading.' % path)
        dlg.Destroy()

    def save(self, evt):
        if not self.current_path:
            error_dialog('filename for save not known')
        else:
            try:
                f = open(self.current_path, 'w')
                f.write(self.setup.assemble_input())
            except IOError, e:
                error_dialog('Error opening file %s for writing.' %
                             self.current_path)

    def saveas(self, evt):
        (dir,style) = saveas_dir_style()  # depends on platform
        dlg = wx.FileDialog(self, message='Save file as ...',
                            defaultDir=dir, style=style)
        # ??? dlg.SetFilterIndex(2)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()      # full path
            try:
                f = open(path, 'w')
                f.write(self.setup.assemble_input())
                self.current_path = path
                self.fmenu.Enable(wx.ID_SAVE, True)
                self.SetTitle(os.path.basename(path) + ' - Prover9/Mace4')
            except IOError, e:
                error_dialog('Error opening file %s for writing.' % path)
            
        dlg.Destroy()

    def get_help(self, evt):
        info = ('\n\ncwd: %s\nargv[0]: %s\npath[0]: %s\nprogram_dir(): %s' %
                (os.getcwd(), sys.argv[0], sys.path[0], program_dir()))

        dlg = wx.MessageDialog(self,
                               'No help yet.' + info,
                               'Help',
                               wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()
        
    def get_about(self, evt):
        dlg = wx.MessageDialog(self,
                               Things_to_do,
                               'About',
                               wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()

    def tooltip_toggle(self, evt):
        enable = self.pref_menu.IsChecked(self.tooltip_id)
        wx.ToolTip.Enable(enable)

    def select_font(self, evt):
        data = wx.FontData()
        data.EnableEffects(True)
        data.SetInitialFont(self.box_font)
        dlg = wx.FontDialog(self, data)
        if dlg.ShowModal() == wx.ID_OK:
            data = dlg.GetFontData()
            font = data.GetChosenFont()
            self.box_font = font
            self.setup.update_font(font)
        dlg.Destroy()

    def setup_toggle(self, evt):
        show = (self.view_menu.GetLabel(self.view_id) == 'Show Setup Panel')
        if show:
            (w,h) = self.saved_client_size
            self.setup.Show(True)
            self.SetClientSize((w,h))
            self.Layout()                # rearrange children
            self.view_menu.Enable(self.run_id, True)
            self.view_menu.SetLabel(self.view_id, 'Hide Setup Panel')
        else:
            self.saved_client_size = self.GetClientSize()
            self.setup.Show(False)
            self.Fit()  # shrink window to fit run panel
            self.view_menu.Enable(self.run_id, False)
            self.view_menu.SetLabel(self.view_id, 'Show Setup Panel')
        self.Show(True)

    def run_toggle(self, evt):
        show = (self.view_menu.GetLabel(self.run_id) == 'Show Run Panel')
        if show:
            (w,h) = self.GetClientSize()
            self.control.Show(True)
            self.SetClientSize((w,h))
            self.Layout()                # rearrange children
            self.view_menu.Enable(self.view_id, True)
            self.view_menu.SetLabel(self.run_id, 'Hide Run Panel')
        else:
            # erase run panel, expand setup panel to fill space
            self.control.Show(False)
            self.setup.SetClientSize(self.GetClientSize())
            self.view_menu.Enable(self.view_id, False)
            self.view_menu.SetLabel(self.run_id, 'Show Run Panel')
        self.Show(True)

# END class Main_frame(Frame)

class MySplashScreen(wx.SplashScreen):
    def __init__(self):

        path = os.path.join(image_dir(),'prover9-splash.gif')
        bmp = wx.Image(path, wx.BITMAP_TYPE_GIF).ConvertToBitmap()
        wx.SplashScreen.__init__(
            self, bmp,wx.SPLASH_CENTRE_ON_SCREEN | wx.SPLASH_NO_TIMEOUT,
            5000, None, -1)

class MyApp(wx.App):
    def OnInit(self):
        frame = Main_frame(None, 'Prover9/Mace4')
        self.SetTopWindow(frame)

        # screen_width  = wx.SystemSettings.GetMetric(wx.SYS_SCREEN_X)
        # screen_height  = wx.SystemSettings.GetMetric(wx.SYS_SCREEN_Y)
        # frame.SetSize((screen_width-200, screen_height-100))

        splash = MySplashScreen()
        splash.Show(True)
        time.sleep(2)
        splash.Destroy()

        frame.SetSize((1000,700))
        frame.Show(True)
    
        return True

# Initialization that does not need wx

os.chdir(os.path.expanduser('~'))  # set current directory to user's home

# Redirect:
#  GTK (console):    True:   prints go to new Frame (multiple to same frame)
#                    False:  prints go to console
#  GTK (no console): True:   prints go to new Frame (multiple to same frame)
#                    False:  prints disappear
#
#  Mac (console):    True:   prints go to new Frame (multiple to same frame)
#                    False:  prints go to console
#  Mac (no console): True:   pops 2 terminals, prints go to new Frame (multiple to same frame)
#                    False:  pops 2 terminals, prints go to terminal

app = MyApp(redirect=False)

app.MainLoop()

